@@ -41,20 +41,7 @@
 #define EEL_DEFAULT_POPUP_MENU_DISPLACEMENT 	2
 #define EEL_STANDARD_CLOSE_WINDOW_CONTROL_KEY 	'w'
 
-/* GtkButton */
-void                  eel_gtk_button_set_padding                      (GtkButton            *button,
-								       int                   pad_amount);
-
 /* signals */
-guint                 eel_gtk_signal_connect_free_data                (GtkObject            *object,
-								       const gchar          *name,
-								       GtkSignalFunc         func,
-								       gpointer              data);
-guint                 eel_gtk_signal_connect_free_data_custom         (GtkObject            *object,
-								       const gchar          *name,
-								       GtkSignalFunc         func,
-								       gpointer              data,
-								       GtkDestroyNotify      destroy_func);
 void                  eel_gtk_signal_connect_full_while_alive         (GtkObject            *object,
 								       const gchar          *name,
 								       GtkSignalFunc         func,
@@ -76,8 +63,6 @@ void                  eel_gtk_widget_set_shown                        (GtkWidget
 gboolean              eel_point_in_allocation                         (const GtkAllocation  *allocation,
 								       int                   x,
 								       int                   y);
-void                  eel_gtk_widget_set_font_by_name                 (GtkWidget            *widget,
-								       const char           *font_name);
 gboolean              eel_point_in_widget                             (GtkWidget            *widget,
 								       int                   x,
 								       int                   y);
@@ -112,7 +97,8 @@ void                  eel_gtk_window_set_initial_geometry             (GtkWindow
 void                  eel_gtk_window_set_initial_geometry_from_string (GtkWindow            *window,
 								       const char           *geometry_string,
 								       guint                 minimum_width,
-								       guint                 minimum_height);
+								       guint                 minimum_height,
+								       gboolean		     ignore_position);
 void                  eel_gtk_window_set_up_close_accelerator         (GtkWindow            *window);
 gboolean              eel_gtk_window_event_is_close_accelerator       (GtkWindow            *window,
 								       GdkEventKey          *event);
@@ -145,13 +131,12 @@ void                  eel_gtk_adjustment_set_value                    (GtkAdjust
 								       float                 value);
 void                  eel_gtk_adjustment_clamp_value                  (GtkAdjustment        *adjustment);
 
-/* Client-side drawing with Pango */
-PangoContext *        eel_gtk_widget_get_pango_ft2_context            (GtkWidget            *widget);
-
 /* GtkTreeView */
 gboolean              eel_gtk_tree_view_cell_is_completely_visible    (GtkTreeView          *tree_view,
 								       GtkTreePath          *path,
 								       GtkTreeViewColumn    *column);
+void                  eel_gtk_tree_view_set_activate_on_single_click  (GtkTreeView               *tree_view,
+								       gboolean                   should_activate);
 
 /* GtkViewport */
 gboolean              eel_gtk_viewport_get_visible_rect               (GtkViewport          *viewport, 
